package com.menagerie.ophelia

import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.DisposableEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.platform.LocalContext
import androidx.datastore.core.DataStore
import androidx.datastore.preferences.core.Preferences
import androidx.datastore.preferences.core.stringPreferencesKey
import com.menagerie.ophelia.sensor.SensorDataManager
import com.menagerie.ophelia.sensor.SensorManagerImpl
import com.menagerie.ophelia.ui.theme.OpheliaTheme
import kotlinx.coroutines.flow.onEach
import kotlinx.coroutines.flow.receiveAsFlow
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.map
import kotlinx.coroutines.launch

enum class Theme {
    Auto,
    Light,
    Dark
}

val themeKey = stringPreferencesKey("theme")

@Composable
fun MainView(
    isLargeScreen: Boolean = false,
    prefs: DataStore<Preferences>,
) {

    val sensorManager = SensorManagerImpl()

    val context = LocalContext.current
    val scope = rememberCoroutineScope()

    DisposableEffect(Unit) {
        val dataManager = SensorDataManager(context)
        dataManager.init()

        val job = scope.launch {
            dataManager.data
                .receiveAsFlow()
                .onEach { sensorManager.listener?.onUpdate(it) }
                .collect()
        }

        onDispose {
            dataManager.cancel()
            job.cancel()
        }
    }

    val currentThemeString by prefs
        .data
        .map {
            it[themeKey] ?: Theme.Dark.name
        }
        .collectAsState(Theme.Dark.name)

    val currentTheme = Theme.valueOf(currentThemeString)

    val isDarkTheme: Boolean? = when (currentTheme) {
        Theme.Dark -> true
        Theme.Light -> false
        Theme.Auto -> null
    }
    OpheliaTheme(isDarkTheme ?: isSystemInDarkTheme()) {
        App(
            sensorManager = sensorManager,
            isLarge = isLargeScreen,
            prefs = prefs,
        )
    }
}